@@ -13,30 +13,31 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-const { GObject, Gtk, Gio, Gdk, GdkPixbuf } = imports.gi
+const { GObject, Gtk, Gio, Gdk, GdkPixbuf, cairo } = imports.gi
 const ngettext = imports.gettext.ngettext
 let Gspell; try { Gspell = imports.gi.Gspell } catch (e) {}
 
 const { alphaColor, isExternalURL } = imports.utils
 const { EpubViewAnnotation } = imports.epubView
+const { sepHeaderFunc } = imports.utils
 
 const highlightColors = ['yellow', 'orange', 'red', 'magenta', 'aqua', 'lime']
 
 const settings = new Gio.Settings({ schema_id: pkg.name })
 
-const AnnotationRow = GObject.registerClass({
+var AnnotationRow = GObject.registerClass({
     GTypeName: 'FoliateAnnotationRow',
     Template: 'resource:///com/github/johnfactotum/Foliate/ui/annotationRow.ui',
     InternalChildren: [
         'annotationSection', 'annotationText', 'annotationNote', 'removeButton'
     ]
 }, class AnnotationRow extends Gtk.ListBoxRow {
-    _init(annotation, epubView) {
+    _init(annotation, epubView, removable = true) {
         super._init()
         this.annotation = annotation
         this._epub = epubView
 
-        this._annotationText.label = annotation.text
+        this._annotationText.label = annotation.text.replace(/\n/g, ' ')
         epubView.getSectionFromCfi(annotation.cfi).then(section =>
             this._annotationSection.label = section.label)
 
@@ -46,7 +47,8 @@ const AnnotationRow = GObject.registerClass({
         this._applyNote()
         annotation.connect('notify::note', this._applyNote.bind(this))
 
-        this._removeButton.connect('clicked', () => this._remove())
+        if (removable) this._removeButton.connect('clicked', () => this._remove())
+        else this._removeButton.hide()
     }
     _applyNote() {
         const note = this.annotation.note
@@ -98,7 +100,8 @@ var ContentsStack = GObject.registerClass({
     InternalChildren: [
         'tocTreeView',
         'annotationsStack', 'annotationsListBox',
-        'bookmarksStack', 'bookmarksListBox', 'bookmarkButton'
+        'bookmarksStack', 'bookmarksListBox', 'bookmarkButton',
+        'annotationsSearchEntry'
     ],
     Signals: {
         'row-activated': { flags: GObject.SignalFlags.RUN_FIRST }
@@ -107,16 +110,13 @@ var ContentsStack = GObject.registerClass({
     _init(params) {
         super._init(params)
 
-        this._annotationsListBox.set_header_func((row) => {
-            if (row.get_index()) row.set_header(new Gtk.Separator())
-        })
-        this._bookmarksListBox.set_header_func((row) => {
-            if (row.get_index()) row.set_header(new Gtk.Separator())
-        })
+        this._annotationsListBox.set_header_func(sepHeaderFunc)
+        this._bookmarksListBox.set_header_func(sepHeaderFunc)
 
         this._tocTreeView.connect('row-activated', () => this._onTocRowActivated())
         this._annotationsListBox.connect('row-activated', this._onAnnotationRowActivated.bind(this))
         this._bookmarksListBox.connect('row-activated', this._onBookmarkRowActivated.bind(this))
+        this._annotationsSearchEntry.connect('search-changed', () => this._updateAnnotations())
     }
     set epub(epub) {
         this._epub = epub
@@ -174,6 +174,25 @@ var ContentsStack = GObject.registerClass({
         this._epub.goTo(row.bookmark.cfi)
         this.emit('row-activated')
     }
+    _updateAnnotations() {
+        let model = this._annotations
+        if (!model) return
+        const query = this._annotationsSearchEntry.text.toLowerCase()
+        if (query) {
+            const results = new Gio.ListStore()
+            const n = model.get_n_items()
+            for (let i = 0; i < n; i++) {
+                const annotation = model.get_item(i)
+                const { text, color, note } = annotation
+                if ([text, color, note]
+                    .some(x => x.toLowerCase().includes(query)))
+                    results.append(annotation)
+            }
+            model = results
+        }
+        this._annotationsListBox.bind_model(model, annotation =>
+            new AnnotationRow(annotation, this._epub))
+    }
     _updateData(annotations, bookmarks) {
         if (annotations) {
             this._annotationsStack.visible_child_name =
@@ -181,9 +200,10 @@ var ContentsStack = GObject.registerClass({
             annotations.connect('items-changed', () => {
                 this._annotationsStack.visible_child_name =
                     annotations.get_n_items() ? 'main' : 'empty'
+                this._updateAnnotations()
             })
-            this._annotationsListBox.bind_model(annotations, annotation =>
-                new AnnotationRow(annotation, this._epub))
+            this._annotations = annotations
+            this._updateAnnotations()
         }
         if (bookmarks) {
             this._bookmarksStack.visible_child_name =
@@ -229,6 +249,9 @@ var FindBox = GObject.registerClass({
         this._inSection.connect('toggled', () => this._onFindEntryActivate())
         this._findTreeView.connect('row-activated', () => this._onFindRowActivated())
     }
+    set height_request(h) {
+        this._findScrolledWindow.height_request = h
+    }
     set epub(epub) {
         this._epub = epub
         this._findTreeView.model = this._epub.findResults
@@ -276,21 +299,45 @@ var FindBox = GObject.registerClass({
     }
 })
 
+const refLabels = {
+    'biblioref': _('Bibliography'),
+    'glossref': _('Glossary'),
+    'noteref': null,
+    'annoref': _('Annotation'), // deprecated
+}
+const noteLabels = {
+    'footnote': _('Footnote'),
+    'endnote': _('Endnote'),
+    'rearnote': _('Endnote'), // deprecated
+    'note': _('Note'), // deprecated
+    'annotation': _('Annotation'), // deprecated
+}
+
 var FootnotePopover = GObject.registerClass({
     GTypeName: 'FoliateFootnotePopover',
     Template: 'resource:///com/github/johnfactotum/Foliate/ui/footnotePopover.ui',
     InternalChildren: [
-        'footnoteLabel', 'controls', 'button'
+        'footnoteLabel', 'label', 'button', 'label'
     ]
 }, class FootnotePopover extends Gtk.Popover {
-    _init(footnote, link, epubView) {
+    _init({ footnote, link, refTypes, noteTypes }, epubView) {
         super._init()
         this._link = link
         this._epub = epubView
         this._footnoteLabel.label = footnote
         this._footnoteLabel.connect('activate-link', this._activateLink.bind(this))
         this._button.connect('clicked', () => this._goToLinkedLocation())
-        if (!link) this._controls.hide()
+        if (!link) this._button.hide()
+
+        const refLabelKey = Object.keys(refLabels).find(x => refTypes.includes(x))
+        const refLabel = refLabels[refLabelKey]
+        if (refLabel) this._label.label = refLabel
+        else {
+            const noteLabelKey = Object.keys(noteLabels).find(x => noteTypes.includes(x))
+            const noteLabel = noteLabels[noteLabelKey]
+            if (noteLabel) this._label.label = noteLabel
+            else if (refLabelKey === 'noteRef') this._label.label = _('Note')
+        }
     }
     popup() {
         super.popup()
@@ -408,39 +455,124 @@ var ImageViewer = GObject.registerClass({
     Template: 'resource:///com/github/johnfactotum/Foliate/ui/imageViewer.ui',
     InternalChildren: [
         'scale', 'paned', 'labelArea', 'image', 'label', 'fileChooser',
-        'copyButton', 'saveAsButton'
+        'invertButton'
     ],
     Properties: {
         pixbuf: GObject.ParamSpec.object('pixbuf', 'pixbuf', 'pixbuf',
             GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, GdkPixbuf.Pixbuf.$gtype),
         alt: GObject.ParamSpec.string('alt', 'alt', 'alt',
-            GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, '')
+            GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, ''),
+        invert: GObject.ParamSpec.boolean('invert', 'invert', 'invert',
+            GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT, false)
     }
-}, class ImageViewer extends Gtk.Window {
+}, class ImageViewer extends Gtk.ApplicationWindow {
     _init(params) {
         super._init(params)
+        this.show_menubar = false
+        this._rotation = 0
+        this._zoom = 1
+        this.bind_property('invert', this._invertButton, 'active',
+            GObject.BindingFlags.BIDIRECTIONAL | GObject.BindingFlags.SYNC_CREATE)
+
+        this.actionGroup = new Gio.SimpleActionGroup()
+        const actions = {
+            'copy': () => this.copy(),
+            'save-as': () => this.saveAs(),
+            'zoom-in': () => this.zoom += 0.25,
+            'zoom-out': () => this.zoom -= 0.25,
+            'zoom-restore': () => this.zoom = 1,
+            'rotate-left': () => this.rotate(90),
+            'rotate-right': () => this.rotate(270),
+            'invert': () => this.set_property('invert', !this.invert),
+            'close': () => this.close(),
+        }
+        Object.keys(actions).forEach(name => {
+            const action = new Gio.SimpleAction({ name })
+            action.connect('activate', actions[name])
+            this.actionGroup.add_action(action)
+        })
+        this.insert_action_group('img', this.actionGroup)
+        const overlay = Gtk.Builder.new_from_resource(
+            '/com/github/johnfactotum/Foliate/ui/shortcutsWindow.ui')
+            .get_object('shortcutsWindow')
+        overlay.section_name = 'image-viewer-shortcuts'
+        this.set_help_overlay(overlay)
+
         const pixbuf = this.pixbuf
         const width = pixbuf.get_width()
         const height = pixbuf.get_height()
         const [windowWidth, windowHeight] = this.transient_for.get_size()
-        this.default_width = Math.max(Math.min(width * 1.5, windowWidth), 320)
+        this.default_width = Math.max(Math.min(width * 1.5, windowWidth), 360)
         this.default_height = Math.max(Math.min(height * 1.5 + 150, windowHeight), 200)
         this._image.set_from_pixbuf(pixbuf)
         if (this.alt) this._label.label = this.alt
         else this._labelArea.hide()
 
-        this._copyButton.connect('clicked', () => this.copy())
-        this._saveAsButton.connect('clicked', () => this.saveAs())
-
         this._scale.connect('format-value', (_, x) => `${Math.round(x * 100)}%`)
         this._scale.connect('value-changed', () => {
-            const zoom = this._scale.get_value()
-            this._image.set_from_pixbuf(pixbuf.scale_simple(
-                width * zoom,
-                height * zoom,
-                GdkPixbuf.InterpType.BILINEAR))
+            this._zoom = this._scale.get_value()
+            this._updateZoom()
+            this._update()
         })
         this._fileChooser.transient_for = this
+        this._updateZoom()
+        this._update()
+    }
+    _updateZoom() {
+        const upper = this._scale.adjustment.upper
+        const lower = this._scale.adjustment.lower
+        this.actionGroup.lookup_action('zoom-in').enabled = this._zoom < upper
+        this.actionGroup.lookup_action('zoom-out').enabled = this._zoom > lower
+        this.actionGroup.lookup_action('zoom-restore').enabled = this._zoom !== 1
+    }
+    _update() {
+        let pixbuf = this.pixbuf
+        const width = pixbuf.get_width()
+        const height = pixbuf.get_height()
+        const zoom = this.zoom
+
+        if (this.invert) {
+            const surface = new cairo.ImageSurface(cairo.Format.ARGB32, width, height)
+            const context = new cairo.Context(surface)
+
+            // paint a whtie background first for transparent images
+            context.setSourceRGBA(1, 1, 1, 1)
+            context.paint()
+
+            // paint the image
+            Gdk.cairo_set_source_pixbuf(context, pixbuf, 0, 0)
+            context.paint()
+
+            // invert
+            context.setOperator(cairo.Operator.DIFFERENCE)
+            context.setSourceRGBA(1, 1, 1, 1)
+            context.paint()
+
+            // no hue-rotate :(
+
+            pixbuf = Gdk.pixbuf_get_from_surface(surface, 0, 0, width, height)
+        }
+        this._image.set_from_pixbuf(pixbuf.scale_simple(
+            width * zoom,
+            height * zoom,
+            GdkPixbuf.InterpType.BILINEAR).rotate_simple(this._rotation))
+    }
+    get zoom() {
+        return this._zoom
+    }
+    set zoom(value) {
+        if (value !== this._scale.get_value()) this._scale.set_value(value)
+    }
+    rotate(degree) {
+        this._rotation = (this._rotation + degree) % 360
+        this._update()
+    }
+    get invert() {
+        return this._invert
+    }
+    set invert(invert) {
+        this._invert = invert
+        if (this._image) this._update()
     }
     show() {
         super.show()
