@@ -17,7 +17,8 @@ pkg.initGettext()
 pkg.initFormat()
 pkg.require({
     'Gio': '2.0',
-    'Gtk': '3.0'
+    'Gtk': '3.0',
+    'WebKit2': '4.0'
 })
 
 const { Gio, Gtk, Gdk, GLib, WebKit2 } = imports.gi
@@ -292,7 +293,7 @@ function main(argv) {
                 if (openHint) print(Math.round(progress / total * 100) + '%')
                 if (progress === total && held) {
                     // add a delay because of debouncing when writing metadata
-                    // this doens't feel like the proper way but it's probably
+                    // this doesn't feel like the proper way but it's probably
                     // the easiest fix
                     setTimeout(() => {
                         application.release()
@@ -313,6 +314,8 @@ function main(argv) {
     })
 
     application.connect('startup', () => {
+        if (Handy && imports.gi.versions.Handy === '1') Handy.init()
+
         viewSettings.bind('prefer-dark-theme', Gtk.Settings.get_default(),
             'gtk-application-prefer-dark-theme', Gio.SettingsBindFlags.DEFAULT)
 
@@ -327,6 +330,7 @@ function main(argv) {
             ['app.quit', ['<ctrl>q']],
             ['app.open', ['<ctrl>o']],
             ['app.preferences', ['<ctrl>comma']],
+            ['app.library', ['<ctrl>h']],
 
             ['lib.list-view', ['<ctrl>1']],
             ['lib.grid-view', ['<ctrl>2']],
@@ -357,17 +361,17 @@ function main(argv) {
             ['view.go-prev', ['p']],
             ['view.go-next', ['n']],
             ['view.go-back', ['<alt>p', '<alt>Left']],
-            ['view.zoom-in', ['plus', 'equal', '<ctrl>plus', '<ctrl>equal']],
-            ['view.zoom-out', ['minus', '<ctrl>minus']],
-            ['view.zoom-restore', ['0', '<ctrl>0']],
+            ['view.zoom-in', ['plus', 'equal', 'KP_Add', 'KP_Equal', '<ctrl>plus', '<ctrl>equal', '<ctrl>KP_Add', '<ctrl>KP_Equal']],
+            ['view.zoom-out', ['minus', 'KP_Subtract', '<ctrl>minus', '<ctrl>KP_Subtract']],
+            ['view.zoom-restore', ['0', 'KP_0', '<ctrl>0', '<ctrl>KP_0']],
             ['view.bookmark', ['<ctrl>d']],
             ['view.clear-cache', ['<ctrl><shift>r']],
 
             ['img.copy', ['<ctrl>c']],
             ['img.save-as', ['<ctrl>s']],
-            ['img.zoom-in', ['plus', 'equal', '<ctrl>plus', '<ctrl>equal']],
-            ['img.zoom-out', ['minus', '<ctrl>minus']],
-            ['img.zoom-restore', ['0', '<ctrl>0']],
+            ['img.zoom-in', ['plus', 'equal', 'KP_Add', 'KP_Equal', '<ctrl>plus', '<ctrl>equal', '<ctrl>KP_Add', '<ctrl>KP_Equal']],
+            ['img.zoom-out', ['minus', 'KP_Subtract', '<ctrl>minus', '<ctrl>KP_Subtract']],
+            ['img.zoom-restore', ['0', 'KP_0', '<ctrl>0', '<ctrl>KP_0']],
             ['img.rotate-left', ['<ctrl>Left']],
             ['img.rotate-right', ['<ctrl>Right']],
             ['img.invert', ['<ctrl>i']],
