@@ -149,7 +149,7 @@ const dispatchLocation = async () => {
 
     // rough estimate of reading time
     // should be reasonable for English and European languages
-    // will be way off for some langauges
+    // will be way off for some languages
     const estimate = endPercentage =>
         (endPercentage - percentage) * book.locations.total
         * CHARACTERS_PER_PAGE
@@ -177,7 +177,7 @@ const dispatchLocation = async () => {
                 location: book.locations.locationFromCfi(location.end.cfi),
                 label: endSection.label
             },
-            sectionHref: startSection.href,
+            sectionHref: endSection.href,
             section: index,
             sectionTotal: book.spine.length,
             locationTotal: book.locations.total,
@@ -355,6 +355,11 @@ open() -> 'book-ready' -> loadLocations()
 */
 
 const open = async (uri, filename, inputType, renderTo, options) => {
+    // new in Epub.js >= 0.3.89 — scripts are blocked by default;
+    // but it needs script to handle events in the iframe,
+    // so scripts really need to be allowed in Epub.js no matter what;
+    // this is fine as, we already block script with CSP and WebKit settings
+    options.allowScriptedContent = true
 
     // force rendering as XHTML
     // if method is 'srcdoc' (default) or `write`, it will be rendered as HTML
